"""Generic GeoJSON feed."""
import logging
from datetime import datetime
from typing import Dict, List, Optional, Tuple

from aio_geojson_client.feed import GeoJsonFeed
from aiohttp import ClientSession
from geojson import FeatureCollection

from .feed_entry import GenericFeedEntry

_LOGGER = logging.getLogger(__name__)


class GenericFeed(GeoJsonFeed[GenericFeedEntry]):
    """Generic GeoJSON feed."""

    def __init__(
        self,
        websession: ClientSession,
        home_coordinates: Tuple[float, float],
        url: str,
        filter_radius: float = None,
    ):
        """Initialise this service."""
        super().__init__(websession, home_coordinates, url, filter_radius=filter_radius)

    def __repr__(self):
        """Return string representation of this feed."""
        return "<{}(home={}, url={}, radius={})>".format(
            self.__class__.__name__,
            self._home_coordinates,
            self._url,
            self._filter_radius,
        )

    def _new_entry(
        self, home_coordinates: Tuple[float, float], feature, global_data: Dict
    ) -> GenericFeedEntry:
        """Generate a new entry."""
        return GenericFeedEntry(home_coordinates, feature)

    def _extract_last_timestamp(
        self, feed_entries: List[GenericFeedEntry]
    ) -> Optional[datetime]:
        """Determine latest (newest) entry from the filtered feed."""
        if feed_entries:
            dates = sorted(
                filter(None, [entry.publication_date for entry in feed_entries]),
                reverse=True,
            )
            if dates:
                return dates[0]
        return None

    def _extract_from_feed(self, feed: FeatureCollection) -> Optional[Dict]:
        """Extract global metadata from feed."""
        return None



"""Feed Manager for Generic GeoJSON feed."""
from typing import Awaitable, Callable, List, Tuple

from aio_geojson_client.feed_manager import FeedManagerBase
from aio_geojson_client.status_update import StatusUpdate
from aiohttp import ClientSession

from .feed import GenericFeed


class GenericFeedManager(FeedManagerBase):
    """Feed Manager for Generic GeoJSON feed."""

    def __init__(
        self,
        websession: ClientSession,
        generate_callback: Callable[[str], Awaitable[None]],
        update_callback: Callable[[str], Awaitable[None]],
        remove_callback: Callable[[str], Awaitable[None]],
        coordinates: Tuple[float, float],
        url: str,
        filter_radius: float = None,
        status_callback: Callable[[StatusUpdate], Awaitable[None]] = None,
    ):
        """Initialize the Generic GeoJSON Feed Manager."""
        feed = GenericFeed(
            websession,
            coordinates,
            url,
            filter_radius=filter_radius,
        )
        super().__init__(
            feed,
            generate_callback,
            update_callback,
            remove_callback,
            status_async_callback=status_callback,
        )



"""Generic GeoJSON feed entry."""
import calendar
import logging
from datetime import datetime
from time import strptime
from typing import Dict, Optional, Tuple

import pytz
from aio_geojson_client.feed_entry import FeedEntry
from geojson import Feature

from .consts import ATTR_GUID, ATTR_ID, ATTR_PUB_DATE, ATTR_TITLE

_LOGGER = logging.getLogger(__name__)


class GenericFeedEntry(FeedEntry):
    """Generic GeoJSON feed entry."""

    def __init__(self, home_coordinates: Tuple[float, float], feature: Feature):
        """Initialise this service."""
        super().__init__(home_coordinates, feature)

    @property
    def title(self) -> str:
        """Return the title of this entry."""
        return self._search_in_properties(ATTR_TITLE)

    @property
    def external_id(self) -> str:
        """Return the external id of this entry."""
        # Find a suitable ID for the provided entry.
        external_id = self._search_in_feature(ATTR_ID)
        if not external_id:
            external_id = self._search_in_properties(ATTR_ID)
        if not external_id:
            external_id = self._search_in_properties(ATTR_GUID)
        if not external_id:
            external_id = self.title
        if not external_id:
            # Use geometry as ID as a fallback.
            external_id = hash(self.coordinates)
        return external_id

    @property
    def publication_date(self) -> datetime:
        """Return the publication date of this entry."""
        publication_date = self._search_in_properties(ATTR_PUB_DATE)
        if publication_date:
            # Parse the date. Example: 15/09/2018 9:31:00 AM
            date_struct = strptime(publication_date, "%d/%m/%Y %I:%M:%S %p")
            publication_date = datetime.fromtimestamp(
                calendar.timegm(date_struct), tz=pytz.utc
            )
        return publication_date

    @property
    def properties(self) -> Optional[Dict]:
        """Return all properties found for this entry."""
        if self._feature and self._feature.properties:
            return self._feature.properties
        return None



import os

from setuptools import find_packages, setup

NAME = "aio_geojson_generic_client"
AUTHOR = "Malte Franken"
AUTHOR_EMAIL = "coding@subspace.de"
DESCRIPTION = "An generic async GeoJSON client library."
URL = "https://github.com/exxamalte/python-aio-geojson-generic-client"

REQUIRES = [
    "aio_geojson_client>=0.17",
    "aiohttp>=3.7.4,<4",
    "pytz>=2019.01",
]


with open("README.md", "r") as fh:
    long_description = fh.read()

HERE = os.path.abspath(os.path.dirname(__file__))
VERSION = {}
with open(os.path.join(HERE, NAME, "__version__.py")) as f:
    exec(f.read(), VERSION)  # pylint: disable=exec-used

setup(
    name=NAME,
    version=VERSION["__version__"],
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    description=DESCRIPTION,
    license="Apache-2.0",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=URL,
    packages=find_packages(exclude=("tests", "tests.*")),
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires=REQUIRES,
)
